package org.rcisoft.baidu.censor.service.impl;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.baidu.aip.contentcensor.AipContentCensor;
import com.baidu.aip.contentcensor.EImgType;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.rcisoft.baidu.censor.dto.CensorResult;
import org.rcisoft.baidu.censor.service.ContentCensorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class ContentCensorServiceImpl implements ContentCensorService {

    /**
     * 百度文本审核，识别审核结果的JSON KEY
     */
    final public static String CENSOR_CONCLUSION_TYPE_KEY = "conclusionType";
    //审核结果
    final public static String CENSOR_CONCLUSION_KEY = "conclusion";
    //请求唯一id
    final public static String CENSOR_LOG_ID_KEY = "log_id";

//    @Resource(name = "commonTextCensorClient")
    @Autowired
    AipContentCensor commonTextCensorClient;

    /**
     * 获取常规文本审核结果
     *
     * @param content 内容
     * @return 百度内容审核JSON
     */
    @Override
    public CensorResult getCommonTextCensorResult(String content) {

        //如果内容为空，则直接返回
        if (content == null || content.isEmpty()) {
            return getCensorResult(null);
        }
        try {
            JSONObject response = commonTextCensorClient.textCensorUserDefined(content);
            return getCensorResult(response);
        } catch (Exception exception) {
            log.error("失败"+exception);
            return getCensorResult(null);
        }
    }

    /**
     * 获取照片审核结果
     *
     * @param imageUrl 图片Url
     * @return 百度图片审核JSON
     */
    @Override
    public CensorResult getImageCensorResult(String imageUrl) {

        //如果内容为空，则直接返回
        if (imageUrl == null || imageUrl.isEmpty()) {
            return getCensorResult(null);
        }

        try {
            JSONObject response = commonTextCensorClient.imageCensorUserDefined(imageUrl, EImgType.URL, null);
            return getCensorResult(response);
        } catch (Exception exception) {
            log.error("失败"+exception);
            return getCensorResult(null);
        }
    }

    /**
     * 获取审核结果
     *
     * @param clientJsonObject 百度审核的JSON字段
     * @return 审核结果
     */
    private CensorResult getCensorResult(JSONObject clientJsonObject) {

        //获取代表审核结果的字段
        //审核结果类型，可取值1.合规，2.不合规，3.疑似，4.审核失败
        int conclusionType = 4;
        List<String> error = new ArrayList<>();
        //如果是null就直接判定为失败
        if (clientJsonObject != null) {
            try {
                conclusionType = clientJsonObject.getInt(CENSOR_CONCLUSION_TYPE_KEY);
                log.info("审核结果"+clientJsonObject.getString(CENSOR_CONCLUSION_KEY));
                log.info("请求唯一id"+clientJsonObject.getString(CENSOR_LOG_ID_KEY));
                boolean isPass = conclusionType == 1;
                if (clientJsonObject.has("data")) {
                    JSONArray array = clientJsonObject.getJSONArray("data");
                    for (int i = 0; i < array.length(); i++) {
                        error.add(array.get(i).toString());
                    }
                }
                return new CensorResult(isPass, error);
            } catch (JSONException e) {
                log.error(""+e);
                String json = "{\"msg\":\"获取审核结果失败\"}";
                return new CensorResult(false, Arrays.asList(json));
            }
        } else
            return new CensorResult(false, Arrays.asList("{\"msg\":\"获取审核结果失败\"}"));

    }

}

